#include <ctype.h>
#include <dirent.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>

/** Implementation of the unix tool 'pidof' from scratch.
 * @param[in] cmdline The commandline to search for.
 * @return The command's PID or zero on error. */
int pidof(char *cmdline)
{
	DIR *proc;
	struct dirent *dent;
	char s[256], *p;
	struct stat st;
	int fd = 0;

	proc = opendir("/proc");
	while ((dent = readdir(proc))) {
		snprintf(s, 255, "/proc/%s", dent->d_name);

		/* check only the /proc/<pid> entries */
		p = dent->d_name;
		while (*p && isdigit(*p++))
			;
		if (*p)
			continue;

		/* since uClibc seems not to support the d_type field in
		 * struct dent (being zero all the time), call stat here */
		if (stat(s, &st) || !S_ISDIR(st.st_mode))
			continue;

		snprintf(s, 255, "/proc/%s/cmdline", dent->d_name);
		if ((fd = open(s, O_RDONLY)) < 0)
			continue;

		/* manual strncmp() of both cmdlines */
		for (p = cmdline; *p; p++) {
			if ((read(fd, s, 1) != 1) || (*p != *s)) {
				close(fd);
				fd = 0;
				break;
			}
		}
		if (fd) {
			close(fd);
			closedir(proc);
			return atoi(dent->d_name);
		}
	}
	closedir(proc);
	return 0;
}
